//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1ns/1ns
module TEST_DRIVE;

reg  [ 3:0] SECT;
reg  [ 2:0] CYL;
reg  [15:0] DIN;
reg         WRITE;
reg         GO;
wire        DONE;
wire [15:0] DOUT;

DISK_DRIVE DUT(SECT, CYL, DIN, WRITE, GO, DONE, DOUT);

//
// Test procedure
//
initial
begin: TEST_PROCEDURE
   integer SECTOR;
   integer CYLINDER;
   integer STAMP;
   integer MAX;
   integer AVG;

   $timeformat(-9, 0, " ns", 5);

   GO = 1'b0;
   #10;

   // Verify every sector
   for (CYLINDER = 0; CYLINDER < 8; CYLINDER = CYLINDER + 1) begin: NEXT_CYL
      reg [15:0] GOOD;
      reg [15:0] DATA;

      $write("Checking cylinder %0d\n", CYLINDER);

      READ(CYLINDER, 4'd7, GOOD);

      // Try to corrupt read-only sector
      WRTE(CYLINDER, 4'd7, ~GOOD);
      READ(CYLINDER, 4'd7, DATA);
      EXPECT(DATA, GOOD, CYLINDER, 4'd7);

      // Check every good sector
      for (SECTOR = 0; SECTOR < 16; SECTOR = SECTOR + 1) begin: NEXT_SECT
         if (!GOOD[SECTOR] || SECTOR == 7) disable NEXT_SECT;

         $write("\tsector %0d\n", SECTOR);

	 WRTE(CYLINDER, SECTOR, 16'hA5F0);
         READ(CYLINDER, SECTOR, DATA);
         EXPECT(DATA, 16'hA5F0, CYLINDER, SECTOR);

	 WRTE(CYLINDER, SECTOR, 16'hF05A);
         READ(CYLINDER, SECTOR, DATA);
         EXPECT(DATA, 16'hF05A, CYLINDER, SECTOR);
      end
   end

   AVG = 0;
   MAX = 0;

   // Perform 25 reads
   WRITE = 1'b0;
   repeat (25) begin
      STAMP = $time;
      GO = 1'b1;
      @ (negedge DONE);
      GO = 1'b0;
      @ (posedge DONE);
      STAMP = $time - STAMP;
      $write("Read seek time  = %t\n", STAMP);
      if (STAMP > MAX) MAX = STAMP;
      AVG = AVG + STAMP;
   end

   // Perform 25 writes
   WRITE = 1'b1;
   repeat (25) begin
      STAMP = $time;
      GO = 1'b1;
      @ (negedge DONE);
      GO = 1'b0;
      @ (posedge DONE);
      STAMP = $time - STAMP;
      $write("Write seek time = %t\n", STAMP);
      if (STAMP > MAX) MAX = STAMP;
      AVG = AVG + STAMP;
   end

   // Report statistics
   AVG = AVG / 50;
   $write("Statistics:\n");
   $write("\tAverage seek time: %t (%0s)\n", AVG,
          (AVG < 10) ? "pass" : "FAIL");
   $write("\tMaximum seek time: %t (%0s)\n", MAX,
          (MAX < 15) ? "pass" : "FAIL");

   $finish;
end

task READ;
   input  [ 2:0] CYLINDER;
   input  [ 3:0] SECTOR;
   output [15:0] DATA;
begin
   CYL   = CYLINDER;
   SECT  = SECTOR;
   WRITE = 1'b0;
   GO = 1'b1;
   @ (negedge DONE);
   GO = 1'b0;
   @ (posedge DONE);
   DATA = DOUT;
end
endtask

task WRTE;
   input [ 2:0] CYLINDER;
   input [ 3:0] SECTOR;
   input [15:0] DATA;
begin
   CYL   = CYLINDER;
   SECT  = SECTOR;
   DIN   = DATA;
   WRITE = 1'b1;
   GO = 1'b1;
   @ (negedge DONE);
   GO = 1'b0;
   @ (posedge DONE);
end
endtask

task EXPECT;
   input [15:0] DATA;
   input [15:0] EXP;
   input [ 2:0] CYLINDER;
   input [ 3:0] SECTOR;
begin
   if (DATA !== EXP) begin
      $write("ERROR: Cyl %h, Sect %h: %h instead of %h\n",
             CYLINDER, SECTOR, DATA, EXP);
   end
end
endtask

endmodule
