//
//	Copyright (c) 2009 HDL Express.
//	All rights reserved.
//
//	E-mail: verilog@hdlexpress.com
//	World Wide Web: http://www.hdlexpress.com	
//

`timescale 1ns/1ns

module CONTROL(NEW_FRAME, SET_NF, FULL, ONE_SAMPLE, CLK, RST);
   input NEW_FRAME;      // NEW_FRAME from control register
   output SET_NF;        // Request for setting NEW_FRAME bit
   input FULL;           // Output FIFO is full
   output ONE_SAMPLE;    // OK to compute 1 sample and add to FIFO
   input CLK;            // Main clock
   input RST;            // Synchronous reset

   reg SET_NF;
   wire ONE_SAMPLE;

   reg [5:0] COUNT;      // Counts that 64 samples/frame are computed

   reg [1:0] STATE;
   parameter IDLE    = 2'b00,   // Waits for SW to finish frame program
	     RUN     = 2'b01,   // Compute samples
	     HANDSHK = 2'b10;   // Wait for NEW_FRAME to be set

always @ (posedge CLK)
begin
   if (RST == 1'b1) begin
      STATE  <= HANDSHK;
      SET_NF <= 1'b1;
   end
   else begin
      case (STATE) // synopsys full_case

       //
       // Wait while the software programs new coefficient.
       // The output FIFO is also slowly drained.
       //
       IDLE: begin
	  if (NEW_FRAME == 1'b0) begin
	     COUNT <= 0;
	     STATE <= RUN;
	  end
       end

       //
       // Compute one output value if the output FIFO is not full.
       // Stop after 64 values and request a new frame from the software.
       //
       RUN: begin
	  if (FULL != 1'b1) begin
	     COUNT <= COUNT + 1;
	     if (COUNT == 6'h3F) begin
		STATE  <= HANDSHK;
		SET_NF <= 1'b1;
	     end
	  end
       end

       //
       // Wait for the processor interface to acknowledge
       // that NEW_FRAME needs to be set.
       //
       HANDSHK: begin
	  if (NEW_FRAME == 1'b1) begin
	     STATE  <= IDLE;
	     SET_NF <= 1'b0;
	  end
       end
       
      endcase
   end
   
end

//
// Compute one sample if (and only if) we are running
// and there is room in the output FIFO
//
assign ONE_SAMPLE = (STATE == RUN && FULL != 1'b1);

endmodule
